from typing import Final

# fmt: off
KEYWORDS: Final[dict[str, str]] = {
    "if":       "IF",
    "else":     "ELSE",
    "while":    "WHILE",
    "for":      "FOR",
    "return":   "RETURN",
    "break":    "BREAK",
    "continue": "CONTINUE",
    "true":     "TRUE",
    "false":    "FALSE",
    "null":     "NULL",
    "import":   "IMPORT",
    "as":       "AS",
    "from":     "FROM",
    "async":    "ASYNC",
    "await":    "AWAIT",
    "class":    "CLASS",
    "fn":       "FUNCTION",
    "public":   "PUBLIC",
}
# fmt: on
